@@ -6,22 +6,30 @@
 
 int quadra_enet(vec &x0,
 		mat &R,
+		mat &xAtxA,
 		vec  xty,
 		vec  sgn_grd,
 		double &pen ,
-		uvec   &null) {
+		uvec   &null,
+		bool   usechol,
+		double tol) {
 
   uword iter = 1; // current iterate
-
+  
   uvec A = find(abs(x0) > ZERO) ; // vector of active variables
   vec theta = -sgn_grd   ; // vector of sign of the solution
-  theta.elem(A)   = signs(x0.elem(A));
-
+  theta.elem(A)   = sign(x0.elem(A));
+  
   // Solving the quadratic problem
-  vec x1 = solve(trimatu(R), solve( trimatl(strans(R)), xty - pen * theta));
-
+  vec x1 ;
+  if (usechol) {
+    x1 = solve(trimatu(R), solve( trimatl(strans(R)), xty - pen * theta));
+  } else {
+    x1 = cg(xAtxA, xty - pen*theta, x0, tol) ;
+  }
+  
   // Check for swapping variables
-  uvec swap = find(abs(signs(x1.elem(A)) - theta.elem(A)) > ZERO);
+  uvec swap = find(abs(sign(x1.elem(A)) - theta.elem(A)) > ZERO);
   if (swap.is_empty()) {
     null = swap ; // this is empty
     x0 = x1;
@@ -42,15 +50,20 @@ int quadra_enet(vec &x0,
 
     A = find(abs(x0) > ZERO) ; // vector of active variables
     theta = -sgn_grd        ; // vector of sign of the solution
-    theta.elem(A)   = signs(x0.elem(A));
+    theta.elem(A)   = sign(x0.elem(A));
 
-    vec x2 = solve(trimatu(R), solve( trimatl(strans(R)), xty - pen * theta));
+    vec x2 ;
+    if (usechol) {
+      x2 = solve(trimatu(R), solve( trimatl(strans(R)), xty - pen * theta));
+    } else {
+      x2 = cg(xAtxA, xty - pen*theta, x1, tol) ;
+    }
     iter++;
 
     // This is the gradient on the active part of the parameters
-    vec grd = -xty + strans(R) * R * x2;
+    vec grd = -xty + xAtxA * x2;
     // if the sign is coherent, keep that one...
-    if (fabs(grd(null[0]) + pen * sign(x2(null[0]))) <= ZERO) {
+    if (fabs(grd(null[0]) + pen * as_scalar(sign(x2(null)))) <= ZERO) {
       null = swap; // this is empty
       x0 = x2 ;
     } else {
@@ -78,7 +91,7 @@ int quadra_breg(vec    &beta,
   uvec I             ; // guys living in between the supremum
   uvec toB           ; // guys reaching the boundary after optimization
   uvec toI           ; // guys leaving the boundary after optimization
-  vec  theta = -signs(grd.elem(B)) ;
+  vec  theta = -sign(grd.elem(B)) ;
 
   vec XX_B   ;
   mat XX     ;
@@ -131,10 +144,10 @@ int quadra_breg(vec    &beta,
     // VARIABLES REACHING THE BOUNDARY
     //
     toB = find(abs(beta) > bound);
-    beta.elem(toB) = bound * signs(beta.elem(toB));
+    beta.elem(toB) = bound * sign(beta.elem(toB));
     B = unique(join_cols(B,toB));
     I = setdiff(all,B);
-    theta = signs(beta.elem(B)); // sign of the guys reaching the supremum
+    theta = sign(beta.elem(B)); // sign of the guys reaching the supremum
   }
 
   grd = -xty + xtx * beta ;
@@ -142,7 +155,7 @@ int quadra_breg(vec    &beta,
   //
   // VARIABLE LEAVING THE BOUDARY
   //
-  toI = find(abs(theta + signs(grd.elem(B))) > zero);
+  toI = find(abs(theta + sign(grd.elem(B))) > zero);
   if (!toI.is_empty()) {
     toI = B.elem(toI);
   }
@@ -191,4 +204,3 @@ uvec setdiff(uvec x, uvec y) {
   }
   return(z);
 }
-
